#ifndef DRAWING_H
#define DRAWING_H

#include "mame.h"
#include "pngdib.h"

#define MAX(a,b) ((a) > (b) ? a : b)
#define MIN(a,b) ((a) <= (b) ? a : b)

// The rotation variable in launcher.h is set to one
// of these constants based on the ini file setting
#define ROTATION_NORMAL 0
#define ROTATION_INVERTED 180
#define ROTATION_ROR 90
#define ROTATION_ROL -90

// The menuType variable in launcher.h is set to one
// of these constants based on the ini file setting
#define MENUTYPE_LISTVIEW 		1
#define MENUTYPE_IMAGEBROWSEVIEW 	2

// The previewPictures variable in launcher.h is set to one
// of these
#define PICTURETYPE_CABINETS		0
#define PICTURETYPE_SCREENSHOTS		1
#define PICTURETYPE_FLYERS		2
#define PICTURETYPE_TITLES	3
#define PICTURETYPE_MARQUEES		4

// Gamelist sort order
#define SORTBY_NAME		0
#define SORTBY_YEAR		1
#define SORTBY_MANUFACTURER		2

// Number of pixels that the game list text be inset from the side of the screen
#define LEFT_PAD 6

// Number of pixels that the images will be inset from the side of the screen
#define IMAGE_PAD 10

// The attract mode video windows -- we gotta control it...
HWND attractAVI;

// Set to the pixel height of the status bar every time it's drawn
int statusBarHeight;

// If set, the image browse mode screen will be redrawn
BOOL redrawBrowseMode;

// Get the width in pixels of the given string
UINT getStringWidth(HDC hdc, const char *text);

// Determine how many rows of text will fit on the screen (in MENUTYPE_LISTVIEW)
int GetNumDisplayRows(HDC hdc, long width, long height, int subtractStatusBarHeight);

// Determine how many images will fit on the screen horizontally (in MENUTYPE_IMAGEBROWSEVIEW)
int GetNumImagesX(HDC hdc, long width, long height, int subtractStatusBarHeight);

// Determine how many images will fit on the screen vertically (in MENUTYPE_IMAGEBROWSEVIEW)
int GetNumImagesY(HDC hdc, long width, long height, int subtractStatusBarHeight);

// Get the height of the font selected in the given HDC
int GetFontHeight(HDC hdc);

// Set the font of the given hdc
BOOL SetFont(HDC hdc, char *face, int size, int rotation, BOOL bold);

// Draw images and the game list to the offscreen buffer
BOOL DrawScreen(HDC hdc, HBITMAP hbitmap);

#endif
