#include "hash.h"

// Linked list stuff
LLNODE *insertLLNodeAfter(void *data, LLNODE *previous) {
	LLNODE *node, *tmp;

	node = (LLNODE *)malloc(sizeof(LLNODE));
	node->data = data;
	
	if (previous) {
		tmp = previous->next;
		previous->next = node;
		node->next = tmp;
	} else {
		node->next = NULL;
	}
	
	return node;
}

void destroyNode(LLNODE *node, LLNODE *prev) {
	if (node) {
		if (prev)
			prev->next = node->next;
		free(node);
		node = NULL;
	}
}

// This does not free the data in the nodes!
void destroyLL(LLNODE *firstNode) {
	if (firstNode) {
		if (firstNode->next) {
			destroyLL(firstNode->next);
			firstNode->next = NULL;
		}
		free(firstNode);
		firstNode = NULL;
	}
}



// Hash table stuff
HASHTABLE *createHT(
	UINT numBuckets,
	UINT (*hashFunction)(const void *item, const UINT numBuckets),
	BOOL (*equals)(void *item1, void *item2)
) {
	HASHTABLE *ht;
	
	if (numBuckets > 0) {
		ht = (HASHTABLE *)malloc(sizeof(HASHTABLE));
		ht->buckets = (LLNODE **)malloc(sizeof(LLNODE *) * numBuckets);
		memset(ht->buckets, 0, sizeof(LLNODE *) * numBuckets);
		ht->numBuckets = numBuckets;
		ht->hashFunction = hashFunction;
		ht->equals = equals;

		return ht;
	} else
		return NULL;
}

void destroyHT(HASHTABLE *ht) {
	UINT i;

	if (ht) {
		for (i=0; i<ht->numBuckets; i++)
			destroyLL(ht->buckets[i]);
		free(ht->buckets);
		free(ht);
	}
}

void addHTItem(void *key, void *value, HASHTABLE *ht) {
	UINT hash;
	HASHELEMENT *he;
	LLNODE *node;

	if (ht && key && value) {
		he = (HASHELEMENT *)malloc(sizeof(HASHELEMENT));
		he->key = key;
		he->value = value;

		hash = ht->hashFunction(key, ht->numBuckets);
		if (!ht->buckets[hash])
			ht->buckets[hash] = insertLLNodeAfter(he, NULL);
		else {
			for (node=ht->buckets[hash]; node->next; node=node->next);
			insertLLNodeAfter(he, node);
		}
	}
}

void *removeHTItem(void *key, HASHTABLE *ht) {
	UINT hash;
	LLNODE *node;
	void *value;

	if (ht && key) {
		hash = ht->hashFunction(key, ht->numBuckets);
		if (ht->equals(((HASHELEMENT *)ht->buckets[hash]->data)->key, key)) {
			node = ht->buckets[hash];
			ht->buckets[hash] = node->next;
			value = ((HASHELEMENT *)node->data)->value;
			free(node->data);
			destroyNode(node, NULL);
		} else {
			for (node=ht->buckets[hash]; node->next; node=node->next) {
				if (ht->equals(((HASHELEMENT *)node->next->data)->key, key)) {
					value = ((HASHELEMENT *)node->next->data)->value;
					free(node->next->data);
					destroyNode(node->next, node);
				}
			}
		}
		return value;
	} else
		return NULL;
}

void *findHTItem(void *key, HASHTABLE *ht) {
	UINT hash;
	LLNODE *node;
	
	if (ht && key) {
		hash = ht->hashFunction(key, ht->numBuckets);
		for (node=ht->buckets[hash]; node; node=node->next) {
			if (ht->equals(((HASHELEMENT *)node->data)->key, key))
				return ((HASHELEMENT *)node->data)->value;
		}
	}
	return NULL;
}
