#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "ini.h"
#include "launcher.h"
#include "drawing.h"

#define INI_BUFFERSIZE 255

char *getIniString(char *appName, char *keyName, char *defaultValue, char *iniFilename) {
	char buffer[INI_BUFFERSIZE];
	char *retval;

	GetPrivateProfileString(appName, keyName, defaultValue, buffer, INI_BUFFERSIZE, iniFilename);
	retval = (char *)malloc(strlen(buffer)+1);
	strcpy(retval, buffer);

	return retval;
}

char *getIniStringOrNull(char *appName, char *keyName, char *iniFilename) {
	char *retval;
	retval = getIniString(appName, keyName, "", iniFilename);
	return ((strlen(retval)>0)?retval:NULL);
}

COLORREF getIniColor(char *appName, char *keyName, char *defaultValue, char *iniFilename) {
	char buffer[INI_BUFFERSIZE];

	GetPrivateProfileString(appName, keyName, defaultValue, buffer, INI_BUFFERSIZE, iniFilename);
	return strtol(buffer, NULL, 16);
}

BOOL getIniBool(char *appName, char *keyName, BOOL defaultValue, char *iniFilename) {
	char buffer[INI_BUFFERSIZE];

	GetPrivateProfileString(appName, keyName, (defaultValue==0)?"0":"1", buffer, INI_BUFFERSIZE, iniFilename);
	return (strcmp(buffer, "0") != 0);
}

int getIniInt(char *appName, char *keyName, char *defaultValue, char *iniFilename) {
	char buffer[INI_BUFFERSIZE];

	GetPrivateProfileString(appName, keyName, defaultValue, buffer, INI_BUFFERSIZE, iniFilename);
	return atoi(buffer);
}

void writeIniInt(char *appName, char *keyName, int newValue, char *iniFilename) {
	char buffer[255];

	sprintf(buffer, "%i", newValue);
	WritePrivateProfileString(appName, keyName, buffer, iniFilename);
}

void LoadINIFile() {
	char buffer[INI_BUFFERSIZE];

	// Settings
	allowMenu = getIniBool("Settings", "AllowMenu", TRUE, iniFilename);							// Allow the menu
	useJoystick = getIniBool("Settings", "UseJoystick", TRUE, iniFilename);						// Use the joystick
	useMameInfoDat = getIniBool("Settings", "UseMameInfoDat", TRUE, iniFilename);				// Should mameinfo.dat info be used
	invertPlayer2Screen = getIniBool("Settings", "InvertPlayer2Screen", FALSE, iniFilename);	// Should screen invert on p2 input
	scrollBorderPct = GetPrivateProfileInt("Settings", "ScrollBorderPct", 20, iniFilename);		// Preview mode width
	selectedGameName = getIniString("System", "SelectedGame", "0", iniFilename);				// Selected game
	mameExe = getIniString("Settings", "Mame.exe", "c:\\fix_the_ini_file.exe", iniFilename);	// Mame.exe
	mameInfoDat = getIniString("Settings", "Mameinfo.dat", ".\\mameinfo.dat", iniFilename);		// Mameinfo.dat
	romsDir = getIniString("Settings", "Roms.dir", "c:\\roms", iniFilename);					// Roms directory
	ssDir = getIniString("Settings", "Snapshot.dir", "c:\\snap", iniFilename);					// Snapshot directory
	titleDir = getIniString("Settings", "Titles.dir", "c:\\titles", iniFilename);					// Titles directory
	flyerDir = getIniString("Settings", "Flyer.dir", "c:\\flyers", iniFilename);				// Flyer directory
	marqueeDir = getIniString("Settings", "Marquee.dir", "c:\\marquees", iniFilename);				// Marquee directory
	cabDir = getIniString("Settings", "Cabinet.dir", "c:\\cabinets", iniFilename);				// Cabinet directory
	attractDir = getIniString("Settings", "Attract.dir", "c:\\attracts", iniFilename);				// Attracts directory
	GetPrivateProfileString("Settings", "Orientation", "Normal", buffer, INI_BUFFERSIZE, iniFilename);	// Orientation
	if (strcmp(buffer, "Inverted") == 0)
		rotation = ROTATION_INVERTED;
	else if (strcmp(buffer, "ROR") == 0)
		rotation = ROTATION_ROR;
	else if (strcmp(buffer, "ROL") == 0)
		rotation = ROTATION_ROL;
	else
		rotation = ROTATION_NORMAL;

	GetPrivateProfileString("Settings", "MenuType", "ListView", buffer,							// Menu Type
		INI_BUFFERSIZE, iniFilename);
	if (strcmp(buffer, "ImageBrowser") == 0)
		menuType = MENUTYPE_IMAGEBROWSEVIEW;
	else
		menuType = MENUTYPE_LISTVIEW;

	GetPrivateProfileString("Settings", "PreviewPictures", "ScreenShots", buffer,				// Preview picture type
		INI_BUFFERSIZE, iniFilename);
	if (strcmp(buffer, "Flyers") == 0)
		previewPictures = PICTURETYPE_FLYERS;
	else if (strcmp(buffer, "Marquees") == 0)
		previewPictures = PICTURETYPE_MARQUEES;
	else if (strcmp(buffer, "Cabinets") == 0)
		previewPictures = PICTURETYPE_CABINETS;
	else if (strcmp(buffer, "Titles") == 0)
		previewPictures = PICTURETYPE_TITLES;
	else
		previewPictures = PICTURETYPE_SCREENSHOTS;

	previewWidth = GetPrivateProfileInt("Settings", "PreviewModeWidth", 184, iniFilename);		// Preview mode width
	previewHeight = GetPrivateProfileInt("Settings", "PreviewModeHeight", 244, iniFilename);	// Preview mode height
	previewSpacer = GetPrivateProfileInt("Settings", "PreviewModeSpacer", 20, iniFilename);		// Preview mode spacer
	previewFontSize = GetPrivateProfileInt("Settings", "PreviewFontSize", 8, iniFilename);		// Preview font size

	// Fonts and colors
	fontFace = getIniString("Settings", "Fontface", "Arial", iniFilename);						// Main font face
	fontSize = GetPrivateProfileInt("Settings", "Fontsize", 18, iniFilename);					// Main font size
	foreColor = getIniColor("Settings", "Fontcolor", "0x000000", iniFilename);					// Main fore color
	backColor = getIniColor("Settings", "Backgroundcolor", "0xFFFFFF", iniFilename);			// Main back color
	highlightForeColor = getIniColor("Settings", "HighlightForecolor", "0x000000", iniFilename);	// Highlighted fore color
	highlightBackColor = getIniColor("Settings", "HighlightBackcolor", "0xFFFFFF", iniFilename);	// Highlighted back color
	menuFontSize = GetPrivateProfileInt("Settings", "MenuFontSize", 12, iniFilename);			// Menu font size (menu uses main font face)

	previewForeColor = getIniColor("Settings", "PreviewFontcolor", "0x000000", iniFilename);	// Main fore color (for preview mode)
	previewBackColor = getIniColor("Settings", "PreviewBackgroundcolor", "0xFFFFFF", iniFilename);	// Main back color (for preview mode)
	previewHighlightForeColor = getIniColor("Settings", "PreviewHighlightForecolor", "0x000000", iniFilename);	// Highlighted fore color (for preview mode)
	previewHighlightBackColor = getIniColor("Settings", "PreviewHighlightBackcolor", "0xFFFFFF", iniFilename);	// Highlighted back color (for preview mode)

	// Rom filters
	GetPrivateProfileString("RomOptions", "SortBy", "Name", buffer,
		INI_BUFFERSIZE, iniFilename);
	if (strcmp(buffer, "Year") == 0)
		sortBy = SORTBY_YEAR;
	else if (strcmp(buffer, "Manufacturer") == 0)
		sortBy = SORTBY_MANUFACTURER;
	else
		sortBy = SORTBY_NAME;

	displayMissingRoms = getIniBool("RomOptions", "DisplayMissingRoms", FALSE, iniFilename);	// Display missing roms
	displayVerticalRoms = getIniBool("RomOptions", "DisplayVertical", TRUE, iniFilename);		// Display vertical roms
	displayHorizontalRoms = getIniBool("RomOptions", "DisplayHorizontal", TRUE, iniFilename);	// Display horizontal roms
	displayClones = getIniBool("RomOptions", "DisplayClones", TRUE, iniFilename);				// Display clones
	displayNeoGeo = getIniBool("RomOptions", "DisplayNeoGeo", TRUE, iniFilename);				// Display neogeo roms
	displayNonNeoGeo = getIniBool("RomOptions", "DisplayNonNeoGeo", TRUE, iniFilename);			// Display non-neogeo roms
	displayVector = getIniBool("RomOptions", "DisplayVector", TRUE, iniFilename);				// Display vector roms
	displayRaster = getIniBool("RomOptions", "DisplayRaster", TRUE, iniFilename);				// Display raster roms
	minPlayers = GetPrivateProfileInt("RomOptions", "MinPlayers", 0, iniFilename);				// Min players
	maxPlayers = GetPrivateProfileInt("RomOptions", "MaxPlayers", 8, iniFilename);				// Max players
	minButtons = GetPrivateProfileInt("RomOptions", "MinButtons", 0, iniFilename);				// Min buttons
	maxButtons = GetPrivateProfileInt("RomOptions", "MaxButtons", 8, iniFilename);				// Max buttons
	minCoins = GetPrivateProfileInt("RomOptions", "MinCoins", 0, iniFilename);					// Min coins
	maxCoins = GetPrivateProfileInt("RomOptions", "MaxCoins", 8, iniFilename);					// Max coins
	controllerTypes = GetPrivateProfileInt("RomOptions", "ControllerTypes", 127, iniFilename);	// Controller types

	// Key mappings
	keySelectUp = getIniInt("Keymappings", "SelectUp", "38", iniFilename);						// Up key
	keySelectDown = getIniInt("Keymappings", "SelectDown", "40", iniFilename);					// Down key
	keySelectPageUp = getIniInt("Keymappings", "SelectPageUp", "37", iniFilename);				// Page up key
	keySelectPageDown = getIniInt("Keymappings", "SelectPageDown", "39", iniFilename);			// Page down key
	keyStartGame = getIniInt("Keymappings", "StartGame", "49", iniFilename);					// Start game
	keyShowMenu = getIniInt("Keymappings", "ShowMenu", "50", iniFilename);						// Show menu
	keyExit = getIniInt("Keymappings", "Exit", "27", iniFilename);								// Exit
	keyGameInfo = getIniInt("Keymappings", "GameInfo", "17", iniFilename);						// Show mameinfo.dat info

	keyP2SelectUp = getIniInt("Keymappings", "P2SelectUp", "82", iniFilename);					// Up key (Player 2)
	keyP2SelectDown = getIniInt("Keymappings", "P2SelectDown", "70", iniFilename);				// Down key (Player 2)
	keyP2SelectPageUp = getIniInt("Keymappings", "P2SelectPageUp", "68", iniFilename);			// Page up key (Player 2)
	keyP2SelectPageDown = getIniInt("Keymappings", "P2SelectPageDown", "71", iniFilename);		// Page down key (Player 2)
	keyP2StartGame = getIniInt("Keymappings", "P2StartGame", "65", iniFilename);				// Start game (Player 2)
	keyP2ShowMenu = getIniInt("Keymappings", "P2ShowMenu", "83", iniFilename);					// Show menu (Player 2)
	keyP2Exit = getIniInt("Keymappings", "P2Exit", "87", iniFilename);							// Exit (Player 2)
	keyP2GameInfo = getIniInt("Keymappings", "P2GameInfo", "81", iniFilename);					// Show mameinfo.dat info (Player 2)

	// Joystick mappings
	joySelectUp = getIniInt("JoystickMappings", "SelectUp", "8", iniFilename);					// Up key
	joySelectDown = getIniInt("JoystickMappings", "SelectDown", "4", iniFilename);				// Down key
	joySelectPageUp = getIniInt("JoystickMappings", "SelectPageUp", "2", iniFilename);			// Page up key
	joySelectPageDown = getIniInt("JoystickMappings", "SelectPageDown", "1", iniFilename);		// Page down key
	joyStartGame = getIniInt("JoystickMappings", "StartGame", "16", iniFilename);				// Start game
	joyShowMenu = getIniInt("JoystickMappings", "ShowMenu", "32", iniFilename);					// Show menu
	joyExit = getIniInt("JoystickMappings", "Exit", "128", iniFilename);						// Exit
	joyGameInfo = getIniInt("JoystickMappings", "GameInfo", "64", iniFilename);					// Show mameinfo.dat info

	// Sounds
	wavStartup = getIniStringOrNull("Sounds", "Startup", iniFilename);							// Startup sound
	wavShutdown = getIniStringOrNull("Sounds", "Shutdown", iniFilename);						// Shutdown sound
	wavStartGame = getIniStringOrNull("Sounds", "StartGame", iniFilename);						// Start mame sound
	wavSelect = getIniStringOrNull("Sounds", "Select", iniFilename);						// Select game sound
	wavError = getIniStringOrNull("Sounds", "Error", iniFilename);								// Error sound

	// Status bar
	showStatusBar = getIniBool("Settings", "ShowStatusBar", TRUE, iniFilename);					// Show the status bar
	statusBarFontFace = getIniString("Settings", "StatusBarFontFace", "Arial", iniFilename);	// Status bar font face
	statusBarFontSize = GetPrivateProfileInt("Settings", "StatusBarFontSize", 12, iniFilename);	// Status bar font size
	statusBarForeColor = getIniColor("Settings", "StatusBarForeColor", "0x000000", iniFilename);	// Status bar fore color
	statusBarBackColor = getIniColor("Settings", "StatusBarBackColor", "0xFFFFFF", iniFilename);	// Status bar back color

}

void SaveINIFile() {
	if (menuType == MENUTYPE_IMAGEBROWSEVIEW)													// Menu Type
		WritePrivateProfileString("Settings", "MenuType", "ImageBrowser", iniFilename);
	else
		WritePrivateProfileString("Settings", "MenuType", "ListView", iniFilename);

	if (previewPictures == PICTURETYPE_FLYERS)													// Preview picture type
		WritePrivateProfileString("Settings", "PreviewPictures", "Flyers", iniFilename);
	else if (previewPictures == PICTURETYPE_SCREENSHOTS)
		WritePrivateProfileString("Settings", "PreviewPictures", "ScreenShots", iniFilename);
	else if (previewPictures == PICTURETYPE_CABINETS)
		WritePrivateProfileString("Settings", "PreviewPictures", "Cabinets", iniFilename);
	else if (previewPictures == PICTURETYPE_TITLES)
		WritePrivateProfileString("Settings", "PreviewPictures", "Titles", iniFilename);
	else
		WritePrivateProfileString("Settings", "PreviewPictures", "Marquees", iniFilename);

	if (gameDisplayList && gameDisplayList[currentGameSelection])								// Selected game
		WritePrivateProfileString("System", "SelectedGame", gameDisplayList[currentGameSelection]->romname, iniFilename);

	WritePrivateProfileString("Settings", "Fontface", fontFace, iniFilename);					// Main font face
	writeIniInt("Settings", "Fontsize", fontSize, iniFilename);									// Main font size
	writeIniInt("Settings", "MenuFontSize", menuFontSize, iniFilename);							// Menu font size
	WritePrivateProfileString("Settings", "StatusBarFontFace", statusBarFontFace, iniFilename);	// Status bar font face
	writeIniInt("Settings", "StatusBarFontSize", statusBarFontSize, iniFilename);				// Status bar font size
	writeIniInt("RomOptions", "DisplayMissingRoms", displayMissingRoms, iniFilename);			// Display missing roms
	writeIniInt("RomOptions", "DisplayVertical", displayVerticalRoms, iniFilename);				// Display vertical roms
	writeIniInt("RomOptions", "DisplayHorizontal", displayHorizontalRoms, iniFilename);			// Display horizontal roms
	writeIniInt("RomOptions", "DisplayClones", displayClones, iniFilename);						// Display clones
	writeIniInt("RomOptions", "DisplayNeoGeo", displayNeoGeo, iniFilename);						// Display neogeo roms
	writeIniInt("RomOptions", "DisplayNonNeoGeo", displayNonNeoGeo, iniFilename);				// Display non-neogeo roms
	writeIniInt("RomOptions", "DisplayVector", displayVector, iniFilename);						// Display vector roms
	writeIniInt("RomOptions", "DisplayRaster", displayRaster, iniFilename);						// Display raster roms
	writeIniInt("RomOptions", "MinPlayers", minPlayers, iniFilename);							// Min players
	writeIniInt("RomOptions", "MaxPlayers", maxPlayers, iniFilename);							// Max players
	writeIniInt("RomOptions", "MinButtons", minButtons, iniFilename);							// Min buttons
	writeIniInt("RomOptions", "MaxButtons", maxButtons, iniFilename);							// Max buttons
	writeIniInt("RomOptions", "MinCoins", minCoins, iniFilename);								// Min coins
	writeIniInt("RomOptions", "MaxCoins", maxCoins, iniFilename);								// Max coins
	writeIniInt("Settings", "ShowStatusBar", showStatusBar, iniFilename);						// Show the status bar

	switch (originalRotation) {																	// Screen orientation
		case ROTATION_NORMAL:
			WritePrivateProfileString("Settings", "Orientation", "Normal", iniFilename); break;
		case ROTATION_INVERTED:
			WritePrivateProfileString("Settings", "Orientation", "Inverted", iniFilename); break;
		case ROTATION_ROR:
			WritePrivateProfileString("Settings", "Orientation", "ROR", iniFilename); break;
		case ROTATION_ROL:
			WritePrivateProfileString("Settings", "Orientation", "ROL", iniFilename); break;
	}
}
