#ifndef LAUNCHER_H
#define LAUNCHER_H

#define VERSION "ULTRAMAME"

#define DEBUG 0			// Make this = 1 or 2 to see debug stuff

// Location of the ini file
#define DEFAULTINIFILENAME ".\\ultramame.ini"

// Milliseconds per screen refresh
// Fiddle with this to make the controls feel right
#define UPDATE_TICKS_MS 100

HINSTANCE hInst;					// Application instance
HWND hwndMain;						// Handle to the main window
HDC hdcMain;						// HDC for the main window (displayed)
HDC hdcOffscreen;					// HDC and HBITMAP for offscreen double buffer image
HBITMAP hbitmapOffscreen;
HDC hdcTile;						// HDC and HBITMAP for tiled images (for speed)
HBITMAP hbitmapTile;

int currentGameSelection;			// Index of the currently selected game in gameDisplayList
int firstGameOnScreen;				// Index of the first game displayed on the screen

char *iniFilename;					// Name of the ini file to process
int originalRotation;				// Stores the original rotation for when the screen flips


// All this stuff is loaded from the ini file
int menuType;						// Type of menu to draw (list or image view)
BOOL useJoystick;					// Should the joystick be used if available
int previewWidth;					// Width of images in image preview mode
int previewHeight;					// Height of images in image preview mode
int previewSpacer;					// Amount of space between images in image preview mode
int previewFontSize;				// Font size for preview mode
int previewPictures;				// What pictures are displayed in preview mode (ss or flyer)
COLORREF previewForeColor;			// Text color (for preview mode)
COLORREF previewBackColor;			// Background color (for preview mode)
COLORREF previewHighlightForeColor;	// Text color of the highlighted entry (for preview mode)
COLORREF previewHighlightBackColor;	// Background color of the highlighted entry (for preview mode)
BOOL allowMenu;						// Should the menu be allowed to be displayed
BOOL invertPlayer2Screen;			// Should the screen be inverted when player 2 moves
BOOL useMameInfoDat;				// Should info be read/displayed from mameinfo.dat
int scrollBorderPct;				// Percentage of screen size used as space in mameinfo screen
int menuFontSize;					// Size of text in the menu
char *selectedGameName;				// Name of last game run
char *fontFace;						// Name of font to use
int fontSize;						// Size of font to use
COLORREF foreColor;					// Text color
COLORREF backColor;					// Background color
COLORREF highlightForeColor;		// Text color of the highlighted entry
COLORREF highlightBackColor;		// Background color of the highlighted entry
int rotation;						// Orientation of the screen (constants in drawing.h)
char *mameExe;						// Location of mame.exe
char *mameInfoDat;					// Location of mameinfo.dat
char *romsDir;						// Directory containing roms
char *ssDir;						// Directory containing screenshots (or whatever)
char *flyerDir;						// Directory containing flyers (or whatever)
char *marqueeDir;				// Directory containing marquees (or whatever)
char *cabDir;				// Directory containing cabinets (or whatever)
char *titleDir;				// Directory title shots (or whatever)
char *attractDir;			// Attract mode videos (or whatever)
int keySelectUp;					// Keycode for scrolling up one entry
int keySelectDown;					// Keycode for scrolling down one entry
int keySelectPageUp;				// Keycode for scrolling up one page
int keySelectPageDown;				// Keycode for scrolling down one page
int keyStartGame;					// Keycode for starting the selected game
int keyShowMenu;					// Keycode for showing the config menu
int keyExit;						// Keycode for exiting the system
int keyGameInfo;					// Keycode for showing game info (from mameinfo.dat)
int keyP2SelectUp;					// Keycodes for player2 controls
int keyP2SelectDown;				// ...
int keyP2SelectPageUp;				// ...
int keyP2SelectPageDown;			// ...
int keyP2StartGame;					// ...
int keyP2ShowMenu;					// ...
int keyP2Exit;						// ...
int keyP2GameInfo;					// ...
int joySelectUp;					// Joystick position for scrolling up one entry
int joySelectDown;					// Joystick position for scrolling down one entry
int joySelectPageUp;				// Joystick position for scrolling up one page
int joySelectPageDown;				// Joystick position for scrolling down one page
int joyStartGame;					// Joystick button for starting the selected game
int joyShowMenu;					// Joystick button for showing the config menu
int joyExit;						// Joystick button for exiting the system
int joyGameInfo;					// Joystick button for showing game info (from mameinfo.dat)
char *wavStartup;					// Wav file to play at startup
char *wavShutdown;					// Wav file to play at shutdown
char *wavStartGame;					// Wav file to play at game launch
char *wavSelect;					// Wav file to play at highlight change
char *wavError;						// Wav file to play if something blows up
BOOL showStatusBar;					// Should the status bar be displayed
char *statusBarFontFace;			// Font to use on the status bar
int statusBarFontSize;				// Size of font on the status bar
COLORREF statusBarForeColor;		// Status bar text color
COLORREF statusBarBackColor;		// Status bar background color
int sortBy;
BOOL displayMissingRoms;			// Should games that don't exist in the filesystem be displayed
BOOL displayVerticalRoms;			// Should vertical orientation games be displayed
BOOL displayHorizontalRoms;			// Should horizontal orientation games be displayed
BOOL displayClones;					// Should clones be displayed
BOOL displayNeoGeo;					// Should neogeo games be displayed
BOOL displayNonNeoGeo;				// Should non-neogeo games be displayed
BOOL displayVector;					// Should vector games be displayed
BOOL displayRaster;					// Should raster games be displayed
short minPlayers;					// Only games with at least this many players will be displayed
short maxPlayers;					// Only games with at most this many players will be displayed
short minButtons;					// Only games with at least this many buttons will be displayed
short maxButtons;					// Only games with at most this many buttons will be displayed
short minCoins;						// Only games with at least this many coin slots will be displayed
short maxCoins;						// Only games with at most this many coin slots will be displayed
short controllerTypes;				// Logical and of controller types to allow (defined in mameparser.h)
// End ini file stuff

// Play a wav file
void PlayWAVFile(char *filename, DWORD sync);

#endif
