#include <windows.h>
#include "mameinfodat.h"
#include "mameparser.h"
#include "launcher.h"
#include "hash.h"
#include <stdio.h>
#include <stdlib.h>

BOOL readGameInfo(FILE *file);

BOOL loadMameInfoDat(char *filename) {
	FILE *file;

	if (useMameInfoDat) {
		if ((file = fopen((char *)filename, "r")) != NULL) {
			if (!readGameInfo(file))
				//printf("Failed to load data from mameinfo.dat!\n");

			fclose(file);
			return TRUE;
		} else {
			//printf("Error opening file: [%s]\n", filename);
			useMameInfoDat = 0;
			return TRUE;
		}
	} else
		return TRUE;
}

BOOL readGameInfo(FILE *file) {
	char l[1024];
	char romname[1024];
	char **lines;
	char *line;
	int lineCount, i, cSearch;
	int state;
	const int STATE_START = 0;
	const int STATE_INFO = 1;
	const int STATE_MAME = 2;
	const int STATE_DRV = 3;
	const int STATE_END = 4;

	state = STATE_START;
	lines = NULL;
	lineCount = 0;

	do {
		if (fgets(l, 1024, file)) {
			if (strlen(l) > 0 && l[0] != '#') {
				l[strlen(l)-1] = 0;
				// Check states
				if (strncmp(l, "$info", 5) == 0) {
					if (state == STATE_START) {
						state = STATE_INFO;
						strcpy(romname, l+6);

#if DEBUG
						if(strlen(romname)==0) printf("blank entry\n");
#endif

						// Added to handle commas and extra gamenames in history.dat
						for(cSearch=0; cSearch<=strlen(romname);cSearch++) {
							if(romname[cSearch] == ',') romname[cSearch] = 0;
						}
#if DEBUG
						printf("found info: [%s]\n", romname);
#endif
					} else {
						printf("invalid state found!!!\n");
						break;
					}
				} else if ( (strncmp(l, "$mame", 5) == 0) || (strncmp(l, "$bio", 4) == 0) ) {
					if (state == STATE_INFO) {
						state = STATE_MAME;
#if DEBUG==2
						printf("found mame or bio\n");
#endif
					} else {
						printf("invalid state found!!!\n");
						break;
					}
				} else if (strncmp(l, "$drv", 4) == 0) {
					if (state == STATE_INFO) {
						state = STATE_DRV;
#if DEBUG==2
						printf("found drv\n");
#endif
					} else {
						printf("invalid state found!!!\n");
						break;
					}
				} else if (strncmp(l, "$end", 4) == 0) {
					if (state == STATE_MAME || state == STATE_DRV) {
						GAME *game;

						state = STATE_START;

						// Put the text in the appropriate game
						game = (GAME *)findHTItem(romname, gameListHT);
						if (game) {
							game->mameInfoLines = lines;
							game->numMameInfoLines = lineCount;
						} else {
							for (i=0; i<lineCount; i++)
								free(lines[i]);
							free(lines);
							lineCount = 0;
							lines = NULL;
						}

						// Free the text if the game isn't found
						// (all drivers will run through here)
						lineCount = 0;
						lines = NULL;
#if DEBUG==2
						printf("found end\n");
#endif
					} else {
						printf("invalid state found!!!\n");
						break;
					}
				} else {	// Line of text found
					if (state == STATE_MAME) {
						line = (char *)malloc(strlen(l)+1);
						strcpy(line, l);
						lines = (char **)realloc(lines, sizeof(char *) * ++lineCount);
						lines[lineCount-1] = line;
#if DEBUG==2
						printf("Found text: [%s]\n", line);
#endif
						// Skip the next blank line of text
						if (strlen(l) > 0)
							fgets(l, 1024, file);
					} else {
#if DEBUG==2
						if (strlen(l) > 0) printf("Found text where it shouldn't be: [%s]\n", l);
#endif
					}
				}
			}
		}
	} while (!feof(file));

	return (state == STATE_START || state == STATE_END);

}
