#include "mameparser.h"
#include "launcher.h"
#include "mameinfodat.h"
#include <stdio.h>
#include <stdlib.h>

GAME *readGame(FILE *file);
char *trim(char *str);

// Make the has for a string
UINT stringHashFunction(const void *item, const UINT numBuckets) {
	char *s;
	UINT hash = 0;

	if (item)
		for(s=(char *)item; *s; s++) hash += *s;

	return hash % numBuckets;
}

// Returns !strcmp() of the two items
BOOL stringEquals(void *item1, void *item2) {
	return !strcmp((char *)item1, (char *)item2);
}

BOOL loadListinfoFile(char *filename) {
	FILE *file;
	GAME *game;
	int i;

	freeGameList();

	gameCount = 0;

	if ((file = fopen((char *)filename, "r")) != NULL) {
		while (game = readGame(file)) {
			gameList = (GAME **)realloc(gameList, sizeof(GAME *) * ++gameCount);
			gameList[gameCount - 1] = game;
#if DEBUG==2
			printf("%i: [%s]\n", gameCount, gameList[gameCount-1]->romname);
#endif
		}

		fclose(file);

		// Create the hash table of games for easy access
		gameListHT = createHT(gameCount, stringHashFunction, stringEquals);
		for (i=0; i<gameCount; i++)
			addHTItem(gameList[i]->romname, gameList[i], gameListHT);

		// Point each clone to its parent
		for (i=0; i<gameCount; i++)
			if (gameList[i]->cloneof)
				gameList[i]->parent = (GAME *)findHTItem(gameList[i]->cloneof, gameListHT);
			else
				gameList[i]->parent = NULL;

		// Load data from mameinfo.dat
		if (useMameInfoDat)
			return loadMameInfoDat(mameInfoDat);
		else
			return TRUE;
	} else {
		printf("Error opening file: [%s]\n", filename);
		return FALSE;
	}
}

GAME *readGame(FILE *file) {
	char line[1024];
	char *line2;
	char trash[1000];
	char value[1000];
	GAME *game;
	ROM *rom;
	VIDEO *video;
	INPUT *input;

	game = NULL;

	do {
		if (fgets(line, 1024, file)) {
			if (strlen(line) > 0)
				line[strlen(line) - 1] = 0;
			line2 = trim(line);

			if (strcmp(line2, "") == 0)
				continue;	// Skip blank lines
			else {
				// Found some text, malloc the game if not done already
				if (!game) {
					game = (GAME *)malloc(sizeof(GAME));
					memset(game, 0, sizeof(GAME));
#if DEBUG==2
					printf("allocating new game\n");
#endif
				}
			}

			if (strcmp(line2, "game (") == 0) {	// Skip beginning of game
#if DEBUG==2
				printf(" found game start\n");
#endif
				continue;
			} else if (strcmp(line2, "resource (") == 0) {	// It's a bios file
				game->isResource = TRUE;
#if DEBUG==2
				printf(" game is a bios\n");
#endif
			} else if (strcmp(line2, ")") == 0) {		// Check for end of game
#if DEBUG==2
				printf(" found game end\n");
#endif
				break;
			} else if (strncmp(line2, "name", 4) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->romname = (char *)malloc(strlen(value) + 1);
				strcpy(game->romname, value);
#if DEBUG==2
				printf("  romname: [%s]\n", game->romname);
#endif
			} else if (strncmp(line2, "description", 11) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->description = (char *)malloc(strlen(value) - 1);
				memset(game->description, 0, strlen(value) - 1);
				strncpy(game->description, &value[1], strlen(value)-2);
#if DEBUG==2
				printf("  description: [%s]\n", game->description);
#endif
			} else if (strncmp(line2, "year", 4) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->year = atoi(value);
#if DEBUG==2
				printf("  year: [%i]\n", game->year);
#endif
			} else if (strncmp(line2, "manufacturer", 12) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->manufacturer = (char *)malloc(strlen(value) - 1);
				memset(game->manufacturer, 0, strlen(value) - 1);
				strncpy(game->manufacturer, &value[1], strlen(value)-2);
#if DEBUG==2
				printf("  manufacturer: [%s]\n", game->manufacturer);
#endif
			} else if (strncmp(line2, "cloneof", 7) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->cloneof = (char *)malloc(strlen(value) + 1);
				strcpy(game->cloneof, value);
#if DEBUG==2
				printf("  cloneof: [%s]\n", game->cloneof);
#endif
			} else if (strncmp(line2, "romof", 5) == 0) {
				sscanf(line2, "%s %[^\n]", trash, value);
				game->romof = (char *)malloc(strlen(value) + 1);
				strcpy(game->romof, value);
#if DEBUG==2
				printf("  romof: [%s]\n", game->romof);
#endif
			} else if (strncmp(line2, "rom (", 5) == 0) {
				rom = (ROM *)malloc(sizeof(ROM));
				memset(rom, 0, sizeof(ROM));
				line2 = trim(line2 + 5);	// skip 'rom ('
				while (sscanf(line2, "%s %s %[^\n]", trash, value, line2) >= 2) {
					if (strcmp(trash, "name") == 0) {
						rom->name = (char *)malloc(strlen(value) + 1);
						strcpy(rom->name, value);
					} else if (strcmp(trash, "size") == 0)
						rom->size = atoi(value);
					else if (strcmp(trash, "crc") == 0)
						rom->crc = strtoul(value, (char **)NULL, 16);
				}
				game->roms = (ROM **)realloc(game->roms, sizeof(ROM) * ++game->romcount);
				game->roms[game->romcount - 1] = rom;
#if DEBUG==2
				printf("  rom n/s/c: [%s], %i, %x\n", game->roms[game->romcount - 1]->name, game->roms[game->romcount - 1]->size, game->roms[game->romcount - 1]->crc);
#endif
			} else if (strncmp(line2, "chip (", 4) == 0) {
			} else if (strncmp(line2, "video (", 5) == 0) {
				video = (VIDEO *)malloc(sizeof(VIDEO));
				memset(video, 0, sizeof(VIDEO));
				line2 = trim(line2 + 7);	// skip 'video ('
				while (sscanf(line2, "%s %s %[^\n]", trash, value, line2) >= 2) {
					if (strcmp(trash, "screen") == 0) {
						video->screen = (char *)malloc(strlen(value) + 1);
						strcpy(video->screen, value);
					} else if (strcmp(trash, "orientation") == 0)
						if (strcmp(value, "vertical") == 0)
							video->orientation = VERTICAL;
						else
							video->orientation = HORIZONTAL;
					else if (strcmp(trash, "x") == 0)
						video->xres = atoi(value);
					else if (strcmp(trash, "y") == 0)
						video->yres = atoi(value);
				}
				game->video = video;
#if DEBUG==2
				printf("  vid s/o/x/y: [%s], %i, %i, %i\n", game->video->screen, game->video->orientation, game->video->xres, game->video->yres);
#endif
			} else if (strncmp(line2, "sound (", 5) == 0) {
			} else if (strncmp(line2, "input (", 5) == 0) {
				input = (INPUT *)malloc(sizeof(INPUT));
				memset(input, 0, sizeof(INPUT));
				input->control = CONTROL_JOY8WAY;	// default to 8 way joy
				line2 = trim(line2 + 7);	// skip 'input ('
				while (sscanf(line2, "%s %s %[^\n]", trash, value, line2) >= 2) {
					if (strcmp(trash, "players") == 0)
						input->players = atoi(value);
					else if (strcmp(trash, "buttons") == 0)
						input->buttons = atoi(value);
					else if (strcmp(trash, "coins") == 0)
						input->coins = atoi(value);
					else if (strcmp(trash, "control") == 0)
						if (strcmp(value, "dial") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "doublejoy8way") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "joy4way") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "joy8way") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "stick") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "paddle") == 0)
							input->control = CONTROL_DIAL;
						else if (strcmp(value, "trackball") == 0)
							input->control = CONTROL_DIAL;
				}
				game->input = input;
#if DEBUG==2
				printf("  input p/b/coin/cntrl: %i, %i, %i, %i\n", game->input->players, game->input->buttons, game->input->coins, game->input->control);
#endif
			} else if (strncmp(line2, "dipswitch (", 9) == 0) {
			} else if (strncmp(line2, "driver (", 6) == 0) {
			} else {
#if DEBUG==2
				printf("  Found unhandled line: [%s]\n", line);
#endif
			}
		}
	} while (!feof(file));

	return game;
}

char *trim(char *str) {
	UINT i;
	for (i=0; i<strlen(str); i++)
		if (!isspace(str[i])) break;
	return &str[i];
}

void freeGameList() {
	int i;

	free(gameDisplayList);
	gameDisplayCount = 0;

	destroyHT(gameListHT);
	gameListHT = NULL;

	if (gameList) {
		for (i=0; i<gameCount; i++)
			freeGame(gameList[i]);
		free(gameList);
	}
	gameList = NULL;
	gameCount = 0;
}

void freeGame(GAME *game) {
	int i;

	if (game) {
		if (game->romname) free(game->romname);
		if (game->description) free(game->description);
		if (game->manufacturer) free(game->manufacturer);
		if (game->cloneof) free(game->cloneof);
		if (game->roms) {
			for (i=0; i<game->romcount; i++)
				if (game->roms[i]) {
					if (game->roms[i]->name) free(game->roms[i]->name);
					free (game->roms[i]);
				}
			free(game->roms);
		}
		if (game->video) {
			if (game->video->screen) free (game->video->screen);
			free (game->video);
		}
		if (game->mameInfoLines) {
			for (i=0; i<game->numMameInfoLines; i++)
				free(game->mameInfoLines[i]);
			free(game->mameInfoLines);
			game->mameInfoLines = NULL;
			game->numMameInfoLines = 0;
		}
		free(game);
	}
}
