#include <windows.h>
#include "message.h"
#include "launcher.h"
#include "drawing.h"

// Number of pixels that the message text will be inset
#define X_INSET 15
#define Y_INSET 15

HWND hwndMessage;			// Handle to the message window
HDC hdcMessage;				// HDC for the message window (displayed)
HDC hdcOffscreenMessage;		// HDC and BITMAP for the offscreen double buffer image
HBITMAP hbitmapOffscreenMessage;
char *message;				// Message to be written to the screen

// Create the message window
BOOL createMessageWindow();

// Handles paint & size messages and nothing else
LRESULT CALLBACK MessageWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

void displayMessage(char *messageText) {
	int screenWidth, screenHeight;
	int textWidth, textHeight;
	int x, y, width, height;	// window move metrics
	
	message = messageText;
	
	// Create the message window if it is not already created
	if (hwndMessage == NULL) {
		// Create the window
		if (createMessageWindow()) {
			// Set the font and stuff
			SetFont(hdcOffscreenMessage, fontFace, fontSize, rotation, FALSE);
			SetTextColor(hdcOffscreenMessage, foreColor);
			SetBkMode(hdcOffscreenMessage, TRANSPARENT);
		} else {
			MessageBox(hwndMain, "Failed to create message window!", "Error", MB_OK);
			return;
		}
	} else
		ShowWindow(hwndMessage, SW_HIDE);	// el hacko
	
	// Resize the window 
	screenWidth = GetSystemMetrics(SM_CXSCREEN);
	screenHeight = GetSystemMetrics(SM_CYSCREEN);
	textWidth = getStringWidth(hdcOffscreenMessage, messageText);
	textHeight = GetFontHeight(hdcOffscreenMessage);
	
	switch (rotation) {
		case ROTATION_NORMAL:
		case ROTATION_INVERTED:
			x = MAX((screenWidth - (textWidth + X_INSET * 2)) / 2, X_INSET);
			y = (screenHeight - (textHeight + Y_INSET * 2)) / 2;
			width = MIN(textWidth + X_INSET * 2, screenWidth - X_INSET * 2);
			height = textHeight + Y_INSET * 2;
			break;
		case ROTATION_ROR:
		case ROTATION_ROL:
			x = (screenWidth - (textHeight + Y_INSET * 2)) / 2;;
			y = MAX((screenHeight - (textWidth + X_INSET * 2)) / 2, X_INSET);
			width = textHeight + Y_INSET * 2;
			height = MIN(textWidth + X_INSET * 2, screenHeight - X_INSET * 2);
			break;
	}
	
	MoveWindow(hwndMessage, x, y, width, height, FALSE);
	
	// Show the window
	ShowWindow(hwndMessage, SW_SHOW);
	UpdateWindow(hwndMessage);
}

BOOL createMessageWindow() {
	WNDCLASS wc;

	hwndMessage = NULL;

	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = (WNDPROC)MessageWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName =  "MessageMenu";
	wc.lpszClassName = "MessageWndClass";
	if(!RegisterClass(&wc)) {
		return FALSE;
	}
	
	hwndMessage = CreateWindow(
		"MessageWndClass", NULL, WS_POPUP | WS_THICKFRAME,
		100, 100, 400, 400,	// these don't matter, it gets resized later
		hwndMain, NULL, hInst, NULL);
	
	return(hwndMessage != NULL);
}

void closeMessage() {
	if (hwndMessage) {
		DestroyWindow(hwndMessage);
		UnregisterClass("MessageWndClass", hInst);
		hwndMessage = NULL;
	}
}

LRESULT CALLBACK MessageWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch(uMsg) {
		case WM_CREATE:
			// get the main dc
			hdcMessage = GetDC(hwnd);
			hdcOffscreenMessage = CreateCompatibleDC(hdcMessage);
			break;
		case WM_DESTROY:
			if (hdcOffscreenMessage)
				DeleteDC(hdcOffscreenMessage);
			if (hdcMessage)
				ReleaseDC(hwnd, hdcMessage);
			if (hbitmapOffscreenMessage)
				DeleteObject(hbitmapOffscreenMessage);
			break;
		case WM_SIZE:
			{
				HBRUSH bgBrush;
				RECT rect;
				long width;
				long height;
				HBITMAP hOldBitmap;
				
				// get the width & height of the window
				GetClientRect(hwnd, &rect);
				width = rect.right - rect.left;
				height = rect.bottom - rect.top;
				
				// create the offscreen dc & bitmap
				hbitmapOffscreenMessage = CreateCompatibleBitmap(hdcMessage, width, height);
				hOldBitmap = (HBITMAP)SelectObject(hdcOffscreenMessage, hbitmapOffscreenMessage);
				DeleteObject(hOldBitmap);
				
				SetFont(hdcOffscreenMessage, fontFace, fontSize, rotation, FALSE);
				SetTextColor(hdcOffscreenMessage, foreColor);
				SetBkMode(hdcOffscreenMessage, TRANSPARENT);
				
				// clear the screen (to the background color
				bgBrush = CreateSolidBrush(backColor);
				FillRect(hdcOffscreenMessage, &rect, bgBrush);
				DeleteObject(bgBrush);
				
				switch (rotation) {
					case ROTATION_NORMAL:
						TextOut(hdcOffscreenMessage, rect.left + X_INSET, rect.top + Y_INSET, message, strlen(message));
						break;
					case ROTATION_INVERTED:
						TextOut(hdcOffscreenMessage, rect.right - X_INSET, rect.bottom - Y_INSET, message, strlen(message));
						break;
					case ROTATION_ROR:
						TextOut(hdcOffscreenMessage, rect.right - X_INSET, rect.top + Y_INSET, message, strlen(message));
						break;
					case ROTATION_ROL:
						TextOut(hdcOffscreenMessage, rect.left + X_INSET, rect.bottom - Y_INSET, message, strlen(message));
						break;
				}
				
				InvalidateRect(hwnd, NULL, FALSE);
				break;
			}
		case WM_PAINT:
			{
				PAINTSTRUCT paintStruct;
				RECT rect;
				BeginPaint(hwnd, &paintStruct);
	
				GetClientRect(hwnd, &rect);
				
				if(hdcOffscreenMessage)
					BitBlt(hdcMessage, rect.left, rect.top, rect.right, rect.bottom, hdcOffscreenMessage, 0, 0, SRCCOPY);
				
				EndPaint(hwnd, &paintStruct);
				break;
			}
		default:
			return(DefWindowProc(hwnd, uMsg, wParam, lParam));
	}
	return 0;
}
