#include <windows.h>
#include "scrolltext.h"
#include "launcher.h"
#include "drawing.h"
#include "input.h"
#include <stdio.h>

LRESULT CALLBACK ScrollWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
int runMessageLoopScroll(HWND hwnd);
void processKeyStrokesScroll(HWND hwnd, BOOL *closed);
char **wordWrapText(HWND hwnd, HDC hdc, const char **lines, const int lineCount, int *newLineCount);
void drawScrollText(HWND hwnd);
void freeScrollText();

HDC scrollOnDC;
HDC scrollOffDC;
HBITMAP scrollOffBitmap;
char **unwrappedText;
int numUnwrappedLines;
char **displayText;
int numTextLines;
int firstDisplayLine;

BOOL displayScrollingTextScreen(char **text, int numLines) {
	WNDCLASS wc;
	HWND hwnd;
	RECT rect;
	int i, scrollBorderX, scrollBorderY;

	if (numLines > 0) {
		unwrappedText = text;
		numUnwrappedLines = numLines;
		firstDisplayLine = 0;

		wc.style = CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc = (WNDPROC)ScrollWndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hInstance = hInst;
		wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
		wc.hCursor = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground = GetStockObject(BLACK_BRUSH);
		wc.lpszMenuName =  "ScrollMenu";
		wc.lpszClassName = "ScrollWndClass";
		if (!RegisterClass(&wc))
			return FALSE;

		GetClientRect(hwndMain, &rect);

		scrollBorderX = scrollBorderPct * rect.right / 100;
		scrollBorderY = scrollBorderPct * rect.bottom / 100;

		hwnd = CreateWindow(
			"ScrollWndClass", NULL, WS_POPUP | WS_THICKFRAME,
			scrollBorderX, scrollBorderY, rect.right - scrollBorderX*2, rect.bottom - scrollBorderY*2,
			hwndMain, NULL, hInst, NULL);

		// Show the window
		ShowWindow(hwnd, SW_SHOW);
		UpdateWindow(hwnd);

		// Clear out any stray key presses
		for (i=0; i<255; i++) GetAsyncKeyState(i);

		runMessageLoopScroll(hwnd);

		DestroyWindow(hwnd);
		UnregisterClass("ScrollWndClass", hInst);
	}

	return TRUE;
}

void freeScrollText() {
	// Cleanup the word-wrapped text
	int i;

	for (i=0; i<numTextLines; i++) {
		free(displayText[i]);
		displayText[i] = NULL;
	}
	free(displayText);
	displayText = NULL;
	numTextLines = 0;
}

LRESULT CALLBACK ScrollWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch(uMsg) {
		case WM_CREATE:
			// get the main dc
			scrollOnDC = GetDC(hwnd);
			scrollOffDC = CreateCompatibleDC(scrollOnDC);
			break;
		case WM_DESTROY:
			if (scrollOffDC)
				DeleteDC(scrollOffDC);
			if (scrollOnDC)
				ReleaseDC(hwnd, scrollOnDC);
			if (scrollOffBitmap)
				DeleteObject(scrollOffBitmap);
			scrollOffDC = NULL;
			scrollOnDC = NULL;
			scrollOffBitmap = NULL;
			freeScrollText();
			break;
		case WM_SIZE:
			{
				RECT rect;
				HBITMAP hOldBitmap;
				int width, height;

				// get the width & height of the window
				GetClientRect(hwnd, &rect);
				width = rect.right - rect.left;
				height = rect.bottom - rect.top;

				scrollOffBitmap = CreateCompatibleBitmap(scrollOnDC, width, height);
				hOldBitmap = (HBITMAP)SelectObject(scrollOffDC, scrollOffBitmap);
				DeleteObject(hOldBitmap);

				SetFont(scrollOffDC, fontFace, fontSize, rotation, FALSE);
				SetTextColor(scrollOffDC, foreColor);
				SetBkMode(scrollOffDC, TRANSPARENT);

				// Word wrap the text (cleanup first)
				freeScrollText();
				displayText = wordWrapText(hwnd, scrollOffDC, (const char **)unwrappedText, numUnwrappedLines, &numTextLines);

				drawScrollText(hwnd);
				InvalidateRect(hwnd, NULL, FALSE);

				break;
			}
		case WM_PAINT:
			{
				PAINTSTRUCT paintStruct;
				RECT rect;
				BeginPaint(hwnd, &paintStruct);

				GetClientRect(hwnd, &rect);

				if(scrollOffDC)
					BitBlt(scrollOnDC, rect.left, rect.top, rect.right, rect.bottom, scrollOffDC, 0, 0, SRCCOPY);
				else printf("SOMETING!\n");

				EndPaint(hwnd, &paintStruct);
				break;
			}
		case WM_SETFOCUS:
			inputEnabled = TRUE;
			break;
		case WM_KILLFOCUS:
			inputEnabled = FALSE;
			break;
		default:
			return(DefWindowProc(hwnd, uMsg, wParam, lParam));
	}

	return 0;
}

void drawScrollText(HWND hwnd) {
	int i, x, y, fontHeight, width, height;
	BOOL stopDrawing;
	RECT rect;
	HBRUSH bgBrush;

	// get the width & height of the window
	GetClientRect(hwnd, &rect);
	width = rect.right - rect.left;
	height = rect.bottom - rect.top;

	// clear the screen (to the background color
	bgBrush = CreateSolidBrush(backColor);
	FillRect(scrollOffDC, &rect, bgBrush);
	DeleteObject(bgBrush);

	fontHeight = GetFontHeight(scrollOffDC);
	stopDrawing = FALSE;

	for (i=0; i + firstDisplayLine < numTextLines; i++) {
		switch (rotation) {
			case ROTATION_NORMAL:
				x = LEFT_PAD;
				y = i * fontHeight + LEFT_PAD;
				if (y > height - fontHeight - LEFT_PAD)
					stopDrawing = TRUE;
				break;
			case ROTATION_INVERTED:
				x = width - LEFT_PAD;
				y = height - LEFT_PAD - i * fontHeight;
				if (y < fontHeight + LEFT_PAD)
					stopDrawing = TRUE;
				break;
			case ROTATION_ROR:
				x = width - i * fontHeight - LEFT_PAD;
				y = LEFT_PAD;
				if (x < fontHeight + LEFT_PAD)
					stopDrawing = TRUE;
				break;
			case ROTATION_ROL:
				x = i * fontHeight + LEFT_PAD;
				y = height - LEFT_PAD;
				if (x > width - fontHeight - LEFT_PAD)
					stopDrawing = TRUE;
				break;
		}
		if (!stopDrawing)
			TextOut(scrollOffDC, x, y, displayText[i + firstDisplayLine], strlen(displayText[i + firstDisplayLine]));
	}
}

int runMessageLoopScroll(HWND hwnd) {
	LARGE_INTEGER start, end, freq;
	LONGLONG update_ticks_pc;
	MSG msg;
	BOOL closed;

	closed = FALSE;

	// Get ticks-per-second of the performance counter.
	//   Note the necessary typecast to a LARGE_INTEGER structure
	if (!QueryPerformanceFrequency((LARGE_INTEGER*)&freq)) {
		MessageBox(NULL, "Hardware doesn't support QueryPerformanceFrequency()!", "Error", MB_OK);
		return -1;
	}

	// Convert milliseconds per move to performance counter units per move.
	update_ticks_pc = UPDATE_TICKS_MS * freq.QuadPart / 1000;

	// Initialize the counter.
	QueryPerformanceCounter((LARGE_INTEGER*)&start);

	// the main message loop begins here
	while (TRUE) {
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
			if (!GetMessage(&msg, NULL, 0, 0) || closed) break;
  			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		QueryPerformanceCounter((LARGE_INTEGER*)&end);

		if ((end.QuadPart - start.QuadPart) >= update_ticks_pc) {
			processKeyStrokesScroll(hwnd, &closed);
			start.QuadPart = end.QuadPart;
		}
	} // End of message loop.

	return msg.wParam;
}

void processKeyStrokesScroll(HWND hwnd, BOOL *closed) {
	BOOL p1Up, p1Down, p1Exit;
	BOOL p2Up, p2Down, p2Exit, p1Info, p2Info;
	int newRotation;

	// Record the states of all the keys
	p1Up = keyPressed(keySelectUp) || joyInPosition(joySelectUp);
	p1Down = keyPressed(keySelectDown) || joyInPosition(joySelectDown);
	p1Exit = keyPressed(keyExit) || joyInPosition(joyExit);
	p2Up = keyPressed(keyP2SelectUp);
	p2Down = keyPressed(keyP2SelectDown);
	p2Exit = keyPressed(keyP2Exit);
	p1Info = keyPressed(keyGameInfo) || joyInPosition(joyGameInfo);
	p2Info = keyPressed(keyP2GameInfo);

	// Flip the screen if requested
	if (invertPlayer2Screen) {
		switch (originalRotation) {
			case ROTATION_NORMAL:
				if (p2Up ||  p2Down || p2Exit)
					newRotation = ROTATION_INVERTED;
				if (p1Up || p1Down || p1Exit)
					newRotation = ROTATION_NORMAL;
				break;
			case ROTATION_INVERTED:
				if (p2Up ||  p2Down || p2Exit)
					newRotation = ROTATION_NORMAL;
				if (p1Up || p1Down || p1Exit)
					newRotation = ROTATION_INVERTED;
				break;
			case ROTATION_ROR:
				if (p2Up ||  p2Down || p2Exit)
					newRotation = ROTATION_ROL;
				if (p1Up || p1Down || p1Exit)
					newRotation = ROTATION_ROR;
				break;
			case ROTATION_ROL:
				if (p2Up ||  p2Down || p2Exit)
					newRotation = ROTATION_ROR;
				if (p1Up || p1Down || p1Exit)
					newRotation = ROTATION_ROL;
				break;
		}
		if (rotation != newRotation) {
			rotation = newRotation;
		}
	}

	if (p1Up || p2Up) {			// Select next lower # item
		firstDisplayLine = MAX(0, firstDisplayLine-1);
		drawScrollText(hwnd);
		InvalidateRect(hwnd, NULL, FALSE);
	}

	if (p1Down || p2Down) {		// Select next higher # item
		firstDisplayLine = MIN(numTextLines-1, firstDisplayLine+1);
		drawScrollText(hwnd);
		InvalidateRect(hwnd, NULL, FALSE);
	}

	if (p1Exit || p2Exit || p1Info || p2Info) {		// Exit the menu
		*closed = TRUE;
	}
}

char **wordWrapText(HWND hwnd, HDC hdc, const char **lines, const int lineCount, int *newLineCount) {
	char **newLines;
	char *line;
	int i, width, nlc;
	UINT start, end;
	RECT rect;
	SIZE size;

	newLines = NULL;
	nlc = 0;

	GetClientRect(hwnd, &rect);

	switch (rotation) {
		case ROTATION_NORMAL:
		case ROTATION_INVERTED:
			width = rect.right - LEFT_PAD - LEFT_PAD;
			break;
		case ROTATION_ROR:
		case ROTATION_ROL:
			width = rect.bottom - LEFT_PAD - LEFT_PAD;
			break;
	}

	for (i=0; i<lineCount; i++) {
		// First test to see if the full line fits
		if (!GetTextExtentPoint32(hdc, lines[i], strlen(lines[i]), &size)) {
			printf("GetTextExtentPoint32 failed in wordWrapText()!");
			*newLineCount = 0;
			return NULL;
		}
		if (size.cx <= width) {
			line = (char *)malloc(strlen(lines[i])+1);
			strcpy(line, lines[i]);
			newLines = (char **)realloc(newLines, sizeof(char *) * ++nlc);
			newLines[nlc-1] = line;
		} else {

			start = 0;
			for (end=1; end<strlen(lines[i]); end++) {
				if (!GetTextExtentPoint32(hdc, lines[i]+start, end - start, &size)) {
					printf("GetTextExtentPoint32 failed in wordWrapText()!");
					*newLineCount = 0;
					return NULL;
				}

				// This is sorta hinky, but it seems to work ok
				if (size.cx > width || end == strlen(lines[i])-1) {
					if (size.cx > width) {
						// Find the last white space
						while (!isspace(lines[i][end]))
							end--;
						if (!GetTextExtentPoint32(hdc, lines[i]+start, end - start, &size)) {
							printf("GetTextExtentPoint32 failed in wordWrapText()!");
							*newLineCount = 0;
							return NULL;
						}
						if (size.cx > width)
							while (!isspace(lines[i][--end]));
					}

					line = (char *)malloc(end-start+1);
					strncpy(line, lines[i]+start, end-start);
					line[end-start] = 0;
					newLines = (char **)realloc(newLines, sizeof(char *) * ++nlc);
					newLines[nlc-1] = line;
					start = end + 1;
				}
			}
		}
	}

	*newLineCount = nlc;
	return newLines;
}
